package cz.cvut.kbss.ear;

import cz.cvut.kbss.ear.model.Chatter;
import cz.cvut.kbss.ear.model.Message;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.json.JsonObject;
import javax.json.spi.JsonProvider;
import javax.websocket.Session;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

// cane be managed by CDI or Spring
//import javax.enterprise.context.ApplicationScoped;
//@ApplicationScoped
@Component
@Scope("singleton")
public class ChatSessionHandler {

    private static final Logger LOGGER = Logger.getLogger(ChatSessionHandler.class.getName());

    private int chatterCount = 0;
    private int messageCount = 0;

    private final Set<Session> sessions = new HashSet<>();
    private final Set<Chatter> chatters = new HashSet<>();
    private final Set<Message> messages = new HashSet<>();

    private static ChatSessionHandler instance;

    private ChatSessionHandler() {
    }

    public static ChatSessionHandler getInstance() {
        if (instance == null) {
            instance = new ChatSessionHandler();
        }
        return instance;
    }

    public void addSession(Session session) {
        sessions.add(session);
        for (Chatter chatter : chatters) {
            JsonObject addMessage = createAddChatter(chatter);
            sendToSession(session, addMessage);
        }
        for (Message message : messages) {
            JsonObject addMessage = createAddMessage(message);
            sendToSession(session, addMessage);
        }
    }

    public void removeSession(Session session) {
        sessions.remove(session);
    }

    public List<Chatter> getChatRooms() {
        return new ArrayList<>(chatters);
    }

    public void addMessage(Message msg) {
        msg.setId(messageCount);
        messages.add(msg);
        messageCount++;
        JsonObject addMessage = createAddMessage(msg);
        sendToAllConnectedSessions(addMessage);
    }

    public void addChatter(Chatter chatter, Session session) {
        chatter.setId(chatterCount);
        chatters.add(chatter);
        chatterCount++;
        JsonObject addMessage = createAddChatter(chatter);
        sendToAllConnectedSessions(addMessage);

        JsonObject setChatter = createSetChatter(chatter);
        sendToSession(session, setChatter);
    }

    public void removeChatter(int id) {
        Chatter chatter = getChatterById(id);
        if (chatter != null) {
            chatters.remove(chatter);
            JsonProvider provider = JsonProvider.provider();
            JsonObject removeMessage = provider.createObjectBuilder()
                    .add("action", "removeChatter")
                    .add("id", id)
                    .build();
            sendToAllConnectedSessions(removeMessage);
        }
    }

    public Chatter getChatterById(int id) {
        for (Chatter chatter : chatters) {
            if (chatter.getId() == id) {
                return chatter;
            }
        }
        return null;
    }

    private JsonObject createSetChatter(Chatter chatter) {
        JsonProvider provider = JsonProvider.provider();
        JsonObject addMessage = provider.createObjectBuilder()
                .add("action", "setChatterId")
                .add("id", chatter.getId())
                .build();
        return addMessage;
    }

    private JsonObject createAddChatter(Chatter chatter) {
        JsonProvider provider = JsonProvider.provider();
        JsonObject addMessage = provider.createObjectBuilder()
                .add("action", "addChatter")
                .add("id", chatter.getId())
                .add("nickName", chatter.getNickName())
                .build();
        return addMessage;
    }

    private JsonObject createAddMessage(Message msg) {
        JsonProvider provider = JsonProvider.provider();
        JsonObject addMessage = provider.createObjectBuilder()
                .add("action", "addMessage")
                .add("id", msg.getId())
                .add("text", msg.getText())
                .add("timestamp", new SimpleDateFormat().format(msg.getTimeStamp()))
                .add("chatterNickName", msg.getChatter().getNickName())
                .build();
        return addMessage;
    }

    private void sendToAllConnectedSessions(JsonObject message) {
        for (Session session : sessions) {
            sendToSession(session, message);
        }
    }

    private void sendToSession(Session session, JsonObject message) {
        LOGGER.log(Level.SEVERE, "Sending to {0} the message {1}", new Object[]{session, message});
        try {
            session.getBasicRemote().sendText(message.toString());
        } catch (IOException ex) {
            sessions.remove(session);
            LOGGER.log(Level.SEVERE, null, ex);
        }
    }
}
