package cz.cvut.kbss.ear;

import cz.cvut.kbss.ear.model.Chatter;
import cz.cvut.kbss.ear.model.Message;
import java.io.StringReader;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.json.Json;
import javax.json.JsonObject;
import javax.json.JsonReader;
import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.ServerEndpoint;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

@Component
@ServerEndpoint("/actions")
public class ChatWebSocketServer {

    private static final Logger LOG = Logger.getLogger(ChatWebSocketServer.class.getName());

    @Autowired
    private ChatSessionHandler handler;
    
    @OnOpen
    public void open(Session session) {
        handler.addSession(session);
    }

    @OnClose
    public void close(Session session) {
        handler.removeSession(session);
    }

    @OnError
    public void onError(Throwable error) {
        LOG.log(Level.SEVERE, null, error);
    }

    @OnMessage
    public void handleMessage(String message, Session session) {
        try (JsonReader reader = Json.createReader(new StringReader(message))) {
            JsonObject jsonMessage = reader.readObject();

            switch (jsonMessage.getString("action")) {
                case "addChatter":
                    LOG.log(Level.INFO, "Adding chatter: {0}", message);
                    Chatter chatter = new Chatter();
                    chatter.setNickName(jsonMessage.getString("nickName"));
                    ChatSessionHandler.getInstance().addChatter(chatter, session);
                    break;
                case "removeChatter":
                    LOG.log(Level.INFO, "Removing chatter: {0}", message);
                    int id = (int) jsonMessage.getInt("id");
                    ChatSessionHandler.getInstance().removeChatter(id);
                    break;
                case "addMessage":
                    LOG.log(Level.INFO, "Adding message: {0}", message);
                    Message msg = new Message();
                    msg.setText(jsonMessage.getString("text"));
                    msg.setTimeStamp(new Date());
                    int chatterId = (int) jsonMessage.getInt("chatterId");
                    msg.setChatter(ChatSessionHandler.getInstance().getChatterById(chatterId));
                    ChatSessionHandler.getInstance().addMessage(msg);
                    break;
                default:
                    break;
            }
        }
    }
}
